from __future__ import annotations

from pydantic import BaseModel, Field

from gmfy.constants import BaseEventAction, BaseEventType, LocaleEnum


class EventData(BaseModel):
    event_type: BaseEventType = Field(...)
    user_id: str = Field(...)
    event_action: BaseEventAction | None = Field(default=None)
    idempotence_key: str | None = Field(default=None)


class PaymentData(BaseModel):
    amount: dict = Field(...)
    confirmation: dict = Field(...)
    user_id: str = Field(...)
    description: str = Field(default=None)
    locale: LocaleEnum = Field(default=LocaleEnum.RU)



from __future__ import annotations

from pydantic import BaseModel, Field

from gmfy.constants import LocaleEnum


class BaseAmount(BaseModel):
    value: float = Field(...)
    currency: str = Field(...)


class BaseConfirmation(BaseModel):
    return_url: str = Field(..., alias="returnUrl")


class BasePayment(BaseModel):
    amount: BaseAmount = Field(...)
    confirmation: BaseConfirmation = Field(...)
    user_id: str = Field(..., alias="userId")
    description: str | None = Field(default=None)
    locale: LocaleEnum = Field(default=LocaleEnum.RU)

    class Config:
        populate_by_name = True



from __future__ import annotations

import importlib
import inspect
import pkgutil
from pathlib import Path


class ClassCollector:
    def __init__(self, base_class: type):
        self.base_class = base_class
        self.classes: list[type] = []
        self.packages = self.get_start_folder()

    def get_start_folder(self) -> list[str]:
        current_path = Path.cwd()
        return [
            item.name
            for item in current_path.iterdir()
            if item.is_dir() and (item / "__init__.py").exists()
        ]

    def collect_classes(self):
        for pack in self.packages:
            package = __import__(pack)
            for loader, module_name, is_pkg in pkgutil.walk_packages(package.__path__):  # noqa: B007
                full_name = f"{pack}.{module_name}"
                module = __import__(full_name, fromlist=[""])
                self._visit_module(module)

    def _visit_module(self, module):
        for name, obj in inspect.getmembers(module):
            if (
                inspect.isclass(obj)
                and obj != self.base_class
                and obj not in self.classes
                and issubclass(obj, self.base_class)
                and not name.startswith("Base")
            ):
                self.classes.append(obj)

    def import_classes(self):
        imported_classes = []
        for cls in self.classes:
            module = importlib.import_module(cls.__module__)
            imported_class = getattr(module, cls.__name__)
            imported_classes.append(imported_class)
        return imported_classes



from __future__ import annotations

import logging
from typing import TYPE_CHECKING

from pydantic import ValidationError

from gmfy.event_managers import EventManager, Events, SingleEventManager
from gmfy.exceptions import GMFYClientError
from gmfy.sync_clients.api_client import GMFYApiClientSync

if TYPE_CHECKING:
    import requests

    from gmfy import BasePayment


class GMFYClientSync:
    def __init__(
        self,
        api_key: str,
        base_url: str,
        verify: bool = True,
        timeout: int = 60,
    ):
        self.api_sync_client = GMFYApiClientSync(api_key, base_url, verify, timeout)

    def create_batch_events(self, events: list[Events]) -> requests.Response:
        try:
            message = f"Creating batch events for {len(events)} events"
            logging.info(message)
            event_manager = EventManager(events)
            event_manager.root = [
                event for event in event_manager.root if event.is_ready
            ]
            response = self.api_sync_client.create_events(event_manager)
        except ValidationError as error:
            error_message = "Validation error while creating events"
            raise GMFYClientError(error_message) from error
        else:
            message = (
                f"Batch events created successfully. "
                f"Status code: {response.status_code}. "
                f"Response: {response.text}"
            )
            logging.info(message)
            return response

    def create_event(self, event: Events) -> requests.Response:
        try:
            message = f"Creating event: {event}"
            logging.info(message)
            event_manager = SingleEventManager(event)
            response = self.api_sync_client.create_event(event_manager)
        except ValidationError as error:
            error_message = "Validation error while creating event"
            raise GMFYClientError(error_message) from error
        else:
            message = (
                f"Event created successfully. "
                f"Status code: {response.status_code}."
                f" Response: {response.text}"
            )
            logging.info(message)
            return response

    def create_payment(self, payment: BasePayment) -> requests.Response:
        try:
            message = f"Creating payment: {payment}"
            logging.info(message)
            response = self.api_sync_client.create_payment(payment)
        except ValidationError as error:
            message = "Validation error while creating payment"
            raise GMFYClientError(message) from error
        else:
            message = (
                f"Payment created successfully. "
                f"Status code: {response.status_code}."
                f" Response: {response.text}"
            )
            logging.info(message)
            return response

    def create_resend_code(self, payment_id: str) -> requests.Response:
        message = f"Creating resend code for payment with id {payment_id}"
        logging.info(message)
        return self.api_sync_client.create_resend_code(payment_id)

    def get_api_version(self) -> requests.Response:
        logging.info("Getting API version")
        return self.api_sync_client.get_version()

    def get_users(self, user_id: str | None = None) -> requests.Response:
        logging.info("Getting users")
        return self.api_sync_client.get_users(user_id)

    def get_rating_top_users(
        self,
        rating_id: str,
        offset: int = 0,
        limit: int = 10,
        sort: str = "ASC",
    ) -> requests.Response:
        message = f"Getting top users in rating with id {rating_id}"
        logging.info(message)
        return self.api_sync_client.get_rating_top_users(
            rating_id,
            params={"offset": offset, "limit": limit, "sort": sort},
        )

    def get_challenge_top_users(
        self,
        challenge_id: str,
        limit: int = 10,
    ) -> requests.Response:
        message = f"Getting top users in challenge with id {challenge_id}"
        logging.info(message)
        params = {"limit": limit}
        return self.api_sync_client.get_challenge_top_users(challenge_id, params=params)

    def get_user_badges(self, user_id: str) -> requests.Response:
        message = f"Getting badges for user with id {user_id}"
        logging.info(message)
        return self.api_sync_client.get_user_badges(user_id)

    def get_notifications(self, user_id: str) -> requests.Response:
        message = f"Getting notifications for user with id {user_id}"
        logging.info(message)
        return self.api_sync_client.get_notifications(user_id)

    def get_payment(self, payment_id: str) -> requests.Response:
        message = f"Getting payment with id {payment_id}"
        logging.info(message)
        return self.api_sync_client.get_payment(payment_id)



from enum import StrEnum


class BaseEventType(StrEnum):
    """
    --- Event type ---

    Example:

    subscription = "subscription"
    subscriber = "subscriber"
    vote = "vote"
    register = "register"
    """


class BaseEventAction(StrEnum):
    """
    --- Event Actions ---

    Example:

    create = "create"
    remove = "remove"
    update = "update"
    delete = "delete"
    """


class LocaleEnum(StrEnum):
    """
    --- Local Enum to determine the payment locale. Not overridden ---
    """

    RU = "RU"
    EN = "EN"



from __future__ import annotations

from functools import reduce
from operator import or_
from typing import TypeAlias

from pydantic import Field, RootModel

from gmfy.base_events import BaseEvent
from gmfy.utils import ClassCollector


def collect_event_classes() -> list[type]:
    collector = ClassCollector(BaseEvent)
    collector.collect_classes()
    imported_classes = collector.import_classes()
    BaseEvent.model_config["children_list"].extend(imported_classes)  # type: ignore[typeddict-item]
    return imported_classes


try:
    collect_event_classes()
    Events: TypeAlias = reduce(or_, BaseEvent.model_config["children_list"])  # type: ignore[typeddict-item, valid-type]
except TypeError as error:
    error_message = (
        "Event classes inherited from BaseEvent were not found in "
        "your project. Ensure you've created your events by inheriting "
        "from BaseEvents."
    )
    raise TypeError(error_message) from error


class EventManager(RootModel):
    root: list[Events] = Field(discriminator="event_type")


class SingleEventManager(RootModel):
    root: Events = Field(discriminator="event_type")



class GMFYApiClientBase:
    def __init__(self, api_key: str, base_url: str):
        self.api_key = api_key
        self.base_url = base_url
        self.headers = {
            "x-api-key": self.api_key,
            "Content-Type": "application/json",
        }
        self.urls = {
            "post_event": f"{self.base_url}v1/events/",
            "post_events": f"{self.base_url}v1/events/batch/",
            "post_payment": f"{self.base_url}v1/payments/",
            "resend_code": f"{self.base_url}v1/payments/{{payment_id}}/resend-code/",
            "get_version": f"{self.base_url}v1/version/",
            "get_user": f"{self.base_url}v1/users/{{user_id}}/",
            "get_users": f"{self.base_url}v1/users/",
            "get_ratings": f"{self.base_url}v1/ratings/{{rating_id}}/top/",
            "get_challenges": f"{self.base_url}v1/challenges/{{challenge_id}}/top/",
            "get_badges": f"{self.base_url}v1/badges/{{user_id}}/",
            "get_notifications": f"{self.base_url}v1/notifications/{{user_id}}/",
            "get_payment": f"{self.base_url}v1/payments/{{payment_id}}/",
        }



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any

import requests

from gmfy.base_api_client import GMFYApiClientBase
from gmfy.exceptions import GMFYClientError

if TYPE_CHECKING:
    from gmfy import BasePayment
    from gmfy.event_managers import EventManager, SingleEventManager


class GMFYApiClientSync(GMFYApiClientBase):
    def __init__(self, api_key: str, base_url: str, verify: bool, timeout: int) -> None:
        super().__init__(api_key, base_url)
        self.session = requests.Session()
        self.session.verify = verify
        self.timeout = timeout
        self.session.headers.update(self.headers)

    def _get(self, url: str | None, **kwargs) -> requests.Response:
        try:
            if url is None:
                error_message = "URL doesn't exist, the request was canceled"
                raise GMFYClientError(error_message)
            params = kwargs.get("params", {})
            response = self.session.get(url, timeout=self.timeout, params=params)
            response.raise_for_status()
        except requests.exceptions.HTTPError as error:
            error_message = f"Bad response status for {url}"
            raise GMFYClientError(error_message) from error
        except (
            requests.ConnectionError,
            requests.Timeout,
            requests.RequestException,
        ) as error:
            error_message = "Network error when fetching gmfy"
            raise GMFYClientError(error_message) from error
        else:
            message = f"Successful GET request to {url}"
            logging.info(message)
            return response

    def _post(self, url: str | None, data: Any) -> requests.Response:
        try:
            if url is None:
                error_message = "URL doesn't exist, the request was canceled"
                raise GMFYClientError(error_message)
            response = self.session.post(url=url, data=data, timeout=self.timeout)
            response.raise_for_status()
        except requests.exceptions.HTTPError as error:
            error_message = f"Bad response status for {url}"
            raise GMFYClientError(error_message) from error
        except (
            requests.ConnectionError,
            requests.Timeout,
            requests.RequestException,
        ) as error:
            error_message = "Network error when fetching gmfy"
            raise GMFYClientError(error_message) from error
        else:
            message = f"Successful POST request to {url}"
            logging.info(message)
            return response

    def create_event(self, event_manager: SingleEventManager) -> requests.Response:
        dumped_event = event_manager.model_dump_json(by_alias=True)
        return self._post(self.urls.get("post_event"), dumped_event)

    def create_events(self, event_manager: EventManager) -> requests.Response:
        dumped_events = event_manager.model_dump_json(by_alias=True)
        return self._post(self.urls.get("post_events"), dumped_events)

    def create_payment(self, payment: BasePayment) -> requests.Response:
        dumped_payment = payment.model_dump_json(by_alias=True)
        return self._post(self.urls.get("post_payment"), dumped_payment)

    def create_resend_code(self, payment_id: str) -> requests.Response:
        return self._post(
            self.urls.get("resend_code").format(payment_id=payment_id),  # type: ignore[union-attr]
            None,
        )

    def get_version(self) -> requests.Response:
        return self._get(self.urls.get("get_version"))

    def get_users(self, user_id: str | None) -> requests.Response:
        if user_id:
            return self._get(self.urls.get("get_user").format(user_id=user_id))  # type: ignore[union-attr]
        return self._get(self.urls.get("get_users"))

    def get_rating_top_users(
        self,
        rating_id: str,
        params: dict[str, int | str],
    ) -> requests.Response:
        return self._get(
            self.urls.get("get_ratings").format(rating_id=rating_id),  # type: ignore[union-attr]
            params=params,
        )

    def get_challenge_top_users(
        self,
        challenge_id: str,
        params: dict[str, int],
    ) -> requests.Response:
        return self._get(
            self.urls.get("get_challenges").format(challenge_id=challenge_id),  # type: ignore[union-attr]
            params=params,
        )

    def get_user_badges(self, user_id: str) -> requests.Response:
        return self._get(self.urls.get("get_badges").format(user_id=user_id))  # type: ignore[union-attr]

    def get_notifications(self, user_id: str) -> requests.Response:
        return self._get(self.urls.get("get_notifications").format(user_id=user_id))  # type: ignore[union-attr]

    def get_payment(self, payment_id: str) -> requests.Response:
        return self._get(self.urls.get("get_payment").format(payment_id=payment_id))  # type: ignore[union-attr]

    def close(self) -> None:
        self.session.close()



from typing import Any

from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    computed_field,
)
from pydantic.alias_generators import to_camel

from gmfy.constants import BaseEventAction, BaseEventType


class BaseEvent(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True,
        alias_generator=to_camel,
        children_list=[],  # type: ignore[typeddict-unknown-key]
    )

    user_id: str
    event_type: BaseEventType

    def __init_subclass__(cls, **kwargs: Any) -> None:
        if not cls.__name__.startswith("Base"):
            cls.model_config["children_list"].append(cls)  # type: ignore[typeddict-item]
        return super().__init_subclass__(**kwargs)

    @computed_field()
    def type(self) -> str:
        return self.event_type

    @property
    def is_ready(self) -> bool:
        return True


class BaseActionEvent(BaseEvent):
    event_action: BaseEventAction = Field(exclude=True)

    @computed_field()
    def type(self) -> str:
        return f"{self.event_action}_{self.event_type}"


class BaseUniqueEvent(BaseEvent):
    idempotence_key: str



import logging


class GMFYClientError(Exception):
    """Base class for exceptions with automatic logging."""

    def __init__(self, message: str):
        super().__init__(message)
        logging.error(message)



from importlib.metadata import PackageNotFoundError, version

from gmfy.base_events import BaseActionEvent, BaseEvent, BaseUniqueEvent
from gmfy.base_payment import BaseAmount, BaseConfirmation, BasePayment
from gmfy.constants import BaseEventAction, BaseEventType, LocaleEnum
from gmfy.data import EventData, PaymentData

try:
    __version__ = version(__name__)
except PackageNotFoundError:
    __version__ = None  # type: ignore[assignment]

__all__ = (
    "BaseEventAction",
    "BaseEventType",
    "BaseEvent",
    "BaseActionEvent",
    "BaseUniqueEvent",
    "LocaleEnum",
    "BasePayment",
    "BaseAmount",
    "BaseConfirmation",
    "EventData",
    "PaymentData",
)
